import java.lang
import java.math
import org.bouncycastle.asn1.x9
import org.bouncycastle.math.ec
import org.bouncycastle.math.ec.custom.sec
import scala
import scala.collection
import scala.util
import scalan
import scalan.primitives
import scorex.crypto.authds.avltree.batch
import special.collection
import special.sigma.impl
import special.sigma.wrappers
import typing
import wrappers.scala
import wrappers.scalan



class AnyValue:
    def tVal(self) -> scalan.RType[typing.Any]: ...
    def value(self) -> typing.Any: ...

class AvlTree:
    def contains(self, key: special.collection.Coll[typing.Any], proof: special.collection.Coll[typing.Any]) -> bool: ...
    def createVerifier(self, proof: special.collection.Coll[typing.Any]) -> 'AvlTreeVerifier': ...
    def digest(self) -> special.collection.Coll[typing.Any]: ...
    def enabledOperations(self) -> int: ...
    def get(self, key: special.collection.Coll[typing.Any], proof: special.collection.Coll[typing.Any]) -> scala.Option[special.collection.Coll[typing.Any]]: ...
    def getMany(self, keys: special.collection.Coll[special.collection.Coll[typing.Any]], proof: special.collection.Coll[typing.Any]) -> special.collection.Coll[scala.Option[special.collection.Coll[typing.Any]]]: ...
    def insert(self, operations: special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], special.collection.Coll[typing.Any]]], proof: special.collection.Coll[typing.Any]) -> scala.Option['AvlTree']: ...
    def isInsertAllowed(self) -> bool: ...
    def isRemoveAllowed(self) -> bool: ...
    def isUpdateAllowed(self) -> bool: ...
    def keyLength(self) -> int: ...
    def remove(self, operations: special.collection.Coll[special.collection.Coll[typing.Any]], proof: special.collection.Coll[typing.Any]) -> scala.Option['AvlTree']: ...
    def update(self, operations: special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], special.collection.Coll[typing.Any]]], proof: special.collection.Coll[typing.Any]) -> scala.Option['AvlTree']: ...
    def updateDigest(self, newDigest: special.collection.Coll[typing.Any]) -> 'AvlTree': ...
    def updateOperations(self, newOperations: int) -> 'AvlTree': ...
    def valueLengthOpt(self) -> scala.Option[typing.Any]: ...

class AvlTreeVerifier:
    def digest(self) -> scala.Option[typing.List[int]]: ...
    def performOneOperation(self, operation: scorex.crypto.authds.avltree.batch.Operation) -> scala.util.Try[scala.Option[typing.List[int]]]: ...
    def treeHeight(self) -> int: ...

class BigInt:
    def $amp(self, that: 'BigInt') -> 'BigInt': ...
    def $bar(self, that: 'BigInt') -> 'BigInt': ...
    def $div(self, that: 'BigInt') -> 'BigInt': ...
    @staticmethod
    def $init$($this: 'BigInt') -> None: ...
    def $minus(self, that: 'BigInt') -> 'BigInt': ...
    def $percent(self, m: 'BigInt') -> 'BigInt': ...
    def $plus(self, that: 'BigInt') -> 'BigInt': ...
    def $times(self, that: 'BigInt') -> 'BigInt': ...
    def add(self, that: 'BigInt') -> 'BigInt': ...
    def compareTo(self, that: 'BigInt') -> int: ...
    def divide(self, that: 'BigInt') -> 'BigInt': ...
    def inverseModQ(self) -> 'BigInt': ...
    def max(self, that: 'BigInt') -> 'BigInt': ...
    def min(self, that: 'BigInt') -> 'BigInt': ...
    def minusModQ(self, other: 'BigInt') -> 'BigInt': ...
    def mod(self, m: 'BigInt') -> 'BigInt': ...
    def modQ(self) -> 'BigInt': ...
    def multModQ(self, other: 'BigInt') -> 'BigInt': ...
    def multiply(self, that: 'BigInt') -> 'BigInt': ...
    def negate(self) -> 'BigInt': ...
    def plusModQ(self, other: 'BigInt') -> 'BigInt': ...
    def remainder(self, that: 'BigInt') -> 'BigInt': ...
    def signum(self) -> int: ...
    def subtract(self, that: 'BigInt') -> 'BigInt': ...
    def toAbs(self) -> 'BigInt': ...
    def toBits(self) -> special.collection.Coll[typing.Any]: ...
    def toByte(self) -> int: ...
    def toBytes(self) -> special.collection.Coll[typing.Any]: ...
    def toInt(self) -> int: ...
    def toLong(self) -> int: ...
    def toShort(self) -> int: ...
    def value(self) -> java.math.BigInteger: ...

class Box:
    @staticmethod
    def $init$($this: 'Box') -> None: ...
    _R0__T = typing.TypeVar('_R0__T')  # <T>
    def R0(self, cT: scalan.RType[_R0__T]) -> scala.Option[_R0__T]: ...
    _R1__T = typing.TypeVar('_R1__T')  # <T>
    def R1(self, cT: scalan.RType[_R1__T]) -> scala.Option[_R1__T]: ...
    _R2__T = typing.TypeVar('_R2__T')  # <T>
    def R2(self, cT: scalan.RType[_R2__T]) -> scala.Option[_R2__T]: ...
    _R3__T = typing.TypeVar('_R3__T')  # <T>
    def R3(self, cT: scalan.RType[_R3__T]) -> scala.Option[_R3__T]: ...
    _R4__T = typing.TypeVar('_R4__T')  # <T>
    def R4(self, cT: scalan.RType[_R4__T]) -> scala.Option[_R4__T]: ...
    _R5__T = typing.TypeVar('_R5__T')  # <T>
    def R5(self, cT: scalan.RType[_R5__T]) -> scala.Option[_R5__T]: ...
    _R6__T = typing.TypeVar('_R6__T')  # <T>
    def R6(self, cT: scalan.RType[_R6__T]) -> scala.Option[_R6__T]: ...
    _R7__T = typing.TypeVar('_R7__T')  # <T>
    def R7(self, cT: scalan.RType[_R7__T]) -> scala.Option[_R7__T]: ...
    _R8__T = typing.TypeVar('_R8__T')  # <T>
    def R8(self, cT: scalan.RType[_R8__T]) -> scala.Option[_R8__T]: ...
    _R9__T = typing.TypeVar('_R9__T')  # <T>
    def R9(self, cT: scalan.RType[_R9__T]) -> scala.Option[_R9__T]: ...
    def bytes(self) -> special.collection.Coll[typing.Any]: ...
    def bytesWithoutRef(self) -> special.collection.Coll[typing.Any]: ...
    def creationInfo(self) -> scala.Tuple2[typing.Any, special.collection.Coll[typing.Any]]: ...
    _executeFromRegister__T = typing.TypeVar('_executeFromRegister__T')  # <T>
    def executeFromRegister(self, regId: int, cT: scalan.RType[_executeFromRegister__T]) -> _executeFromRegister__T: ...
    _getReg__T = typing.TypeVar('_getReg__T')  # <T>
    def getReg(self, i: int, cT: scalan.RType[_getReg__T]) -> scala.Option[_getReg__T]: ...
    def id(self) -> special.collection.Coll[typing.Any]: ...
    def propositionBytes(self) -> special.collection.Coll[typing.Any]: ...
    def registers(self) -> special.collection.Coll[AnyValue]: ...
    def toString(self) -> java.lang.String: ...
    def tokens(self) -> special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]: ...
    def value(self) -> int: ...

class Context:
    def HEIGHT(self) -> int: ...
    def INPUTS(self) -> special.collection.Coll[Box]: ...
    def LastBlockUtxoRootHash(self) -> AvlTree: ...
    def OUTPUTS(self) -> special.collection.Coll[Box]: ...
    def SELF(self) -> Box: ...
    def activatedScriptVersion(self) -> int: ...
    def builder(self) -> 'SigmaDslBuilder': ...
    def dataInputs(self) -> special.collection.Coll[Box]: ...
    _getVar__T = typing.TypeVar('_getVar__T')  # <T>
    def getVar(self, id: int, cT: scalan.RType[_getVar__T]) -> scala.Option[_getVar__T]: ...
    def headers(self) -> special.collection.Coll['Header']: ...
    def minerPubKey(self) -> special.collection.Coll[typing.Any]: ...
    def preHeader(self) -> 'PreHeader': ...
    def selfBoxIndex(self) -> int: ...
    def vars(self) -> special.collection.Coll[AnyValue]: ...

class CostModel:
    def AccessAvlTree(self) -> int: ...
    def AccessBox(self) -> int: ...
    def AccessKiloByteOfData(self) -> int: ...
    def CollectionConst(self) -> int: ...
    def DeserializeRegister(self) -> int: ...
    def DeserializeVar(self) -> int: ...
    def GetRegister(self) -> int: ...
    def GetVar(self) -> int: ...
    def PubKeySize(self) -> int: ...
    def SelectField(self) -> int: ...

class CostedObjects:
    @staticmethod
    def $init$($this: 'CostedObjects') -> None: ...
    class SizeAnyValue(special.collection.Sizes.Size['SigmaDsl.AnyValue']):
        def tVal(self) -> scalan.Base.Ref[wrappers.scalan.WRTypes.WRType[typing.Any]]: ...
        def valueSize(self) -> scalan.Base.Ref[special.collection.Sizes.Size[typing.Any]]: ...
    class SizeAnyValueCompanion: ...
    class SizeBox(special.collection.Sizes.Size['SigmaDsl.Box']):
        def bytes(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]]: ...
        def bytesWithoutRef(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]]: ...
        _getReg__T = typing.TypeVar('_getReg__T')  # <T>
        def getReg(self, id: scalan.Base.Ref[typing.Any], tT: scalan.TypeDescs.Elem[_getReg__T]) -> scalan.Base.Ref[special.collection.Sizes.Size[wrappers.scala.WOptions.WOption[_getReg__T]]]: ...
        def propositionBytes(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]]: ...
        def registers(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[wrappers.scala.WOptions.WOption['SigmaDsl.AnyValue']]]]: ...
        def tokens(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[scala.Tuple2[special.collection.Colls.Coll[typing.Any], typing.Any]]]]: ...
    class SizeBoxCompanion: ...
    class SizeBuilder(scalan.Base.Def['CostedObjects.SizeBuilder']):
        def mkSizeAnyValue(self, tVal: scalan.Base.Ref[wrappers.scalan.WRTypes.WRType[typing.Any]], valueSize: scalan.Base.Ref[special.collection.Sizes.Size[typing.Any]]) -> scalan.Base.Ref['CostedObjects.SizeAnyValue']: ...
        def mkSizeBox(self, propositionBytes: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]], bytes: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]], bytesWithoutRef: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]], registers: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[wrappers.scala.WOptions.WOption['SigmaDsl.AnyValue']]]], tokens: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[scala.Tuple2[special.collection.Colls.Coll[typing.Any], typing.Any]]]]) -> scalan.Base.Ref['CostedObjects.SizeBox']: ...
        def mkSizeContext(self, outputs: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]], inputs: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]], dataInputs: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]], selfBox: scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.Box']], lastBlockUtxoRootHash: scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.AvlTree']], headers: scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Header']]], preHeader: scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.PreHeader']], vars: scalan.Base.Ref[special.collection.Colls.Coll[special.collection.Sizes.Size['SigmaDsl.AnyValue']]]) -> scalan.Base.Ref['CostedObjects.SizeContext']: ...
    class SizeBuilderCompanion: ...
    class SizeContext(special.collection.Sizes.Size['SigmaDsl.Context']):
        def dataInputs(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]]: ...
        _getVar__T = typing.TypeVar('_getVar__T')  # <T>
        def getVar(self, id: scalan.Base.Ref[typing.Any], tT: scalan.TypeDescs.Elem[_getVar__T]) -> scalan.Base.Ref[special.collection.Sizes.Size[wrappers.scala.WOptions.WOption[_getVar__T]]]: ...
        def headers(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Header']]]: ...
        def inputs(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]]: ...
        def lastBlockUtxoRootHash(self) -> scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.AvlTree']]: ...
        def outputs(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll['SigmaDsl.Box']]]: ...
        def preHeader(self) -> scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.PreHeader']]: ...
        def selfBox(self) -> scalan.Base.Ref[special.collection.Sizes.Size['SigmaDsl.Box']]: ...
    class SizeContextCompanion: ...
    class SizeSigmaProp(special.collection.Sizes.Size['SigmaDsl.SigmaProp']):
        def propBytes(self) -> scalan.Base.Ref[special.collection.Sizes.Size[special.collection.Colls.Coll[typing.Any]]]: ...
    class SizeSigmaPropCompanion: ...

class CostedObjectsModule(special.sigma.impl.CostedObjectsDefs): ...

class DslSyntaxExtensions:
    special$sigma$DslSyntaxExtensions$$dsl: 'SigmaDslBuilder' = ...
    def __init__(self, dsl: 'SigmaDslBuilder'): ...
    class BooleanOps:
        $outer: 'DslSyntaxExtensions' = ...
        def __init__(self, $outer: 'DslSyntaxExtensions', source: bool): ...
        def $amp$amp(self, prop: 'SigmaProp') -> 'SigmaProp': ...
        def $bar$bar(self, prop: 'SigmaProp') -> 'SigmaProp': ...

class Extensions:
    @staticmethod
    def showECPoint(p: org.bouncycastle.math.ec.ECPoint) -> java.lang.String: ...
    class GroupElementOps:
        def __init__(self, source: 'GroupElement'): ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def showToString(self) -> java.lang.String: ...
        def source(self) -> 'GroupElement': ...
    class GroupElementOps$:
        MODULE$: typing.ClassVar['Extensions.GroupElementOps.'] = ...
        def __init__(self): ...
        def equals$extension(self, $this: 'GroupElement', x$1: typing.Any) -> bool: ...
        def hashCode$extension(self, $this: 'GroupElement') -> int: ...
        def showToString$extension(self, $this: 'GroupElement') -> java.lang.String: ...

class GroupElement:
    def exp(self, k: BigInt) -> 'GroupElement': ...
    def getEncoded(self) -> special.collection.Coll[typing.Any]: ...
    def isInfinity(self) -> bool: ...
    def multiply(self, that: 'GroupElement') -> 'GroupElement': ...
    def negate(self) -> 'GroupElement': ...
    def value(self) -> org.bouncycastle.math.ec.ECPoint: ...

class Header:
    def ADProofsRoot(self) -> special.collection.Coll[typing.Any]: ...
    def extensionRoot(self) -> special.collection.Coll[typing.Any]: ...
    def height(self) -> int: ...
    def id(self) -> special.collection.Coll[typing.Any]: ...
    def minerPk(self) -> GroupElement: ...
    def nBits(self) -> int: ...
    def parentId(self) -> special.collection.Coll[typing.Any]: ...
    def powDistance(self) -> BigInt: ...
    def powNonce(self) -> special.collection.Coll[typing.Any]: ...
    def powOnetimePk(self) -> GroupElement: ...
    def stateRoot(self) -> AvlTree: ...
    def timestamp(self) -> int: ...
    def transactionsRoot(self) -> special.collection.Coll[typing.Any]: ...
    def version(self) -> int: ...
    def votes(self) -> special.collection.Coll[typing.Any]: ...

class InvalidType(java.lang.Exception):
    def __init__(self, message: typing.Union[java.lang.String, str]): ...

class PreHeader:
    def height(self) -> int: ...
    def minerPk(self) -> GroupElement: ...
    def nBits(self) -> int: ...
    def parentId(self) -> special.collection.Coll[typing.Any]: ...
    def timestamp(self) -> int: ...
    def version(self) -> int: ...
    def votes(self) -> special.collection.Coll[typing.Any]: ...

class SigmaContract:
    @staticmethod
    def $init$($this: 'SigmaContract') -> None: ...
    _Collection__T = typing.TypeVar('_Collection__T')  # <T>
    def Collection(self, items: scala.collection.Seq[_Collection__T], cT: scalan.RType[_Collection__T]) -> special.collection.Coll[_Collection__T]: ...
    def allOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def allZK(self, conditions: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def anyOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def anyZK(self, conditions: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def atLeast(self, bound: int, props: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def blake2b256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def builder(self) -> 'SigmaDslBuilder': ...
    def byteArrayToBigInt(self, bytes: special.collection.Coll[typing.Any]) -> BigInt: ...
    def byteArrayToLong(self, bytes: special.collection.Coll[typing.Any]) -> int: ...
    def decodePoint(self, encoded: special.collection.Coll[typing.Any]) -> GroupElement: ...
    def groupGenerator(self) -> GroupElement: ...
    def longToByteArray(self, l: int) -> special.collection.Coll[typing.Any]: ...
    def proveDHTuple(self, g: GroupElement, h: GroupElement, u: GroupElement, v: GroupElement) -> 'SigmaProp': ...
    def proveDlog(self, g: GroupElement) -> 'SigmaProp': ...
    def sha256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def sigmaProp(self, b: bool) -> 'SigmaProp': ...
    _substConstants__T = typing.TypeVar('_substConstants__T')  # <T>
    def substConstants(self, scriptBytes: special.collection.Coll[typing.Any], positions: special.collection.Coll[typing.Any], newValues: special.collection.Coll[_substConstants__T]) -> special.collection.Coll[typing.Any]: ...
    def verifyZK(self, cond: scala.Function0['SigmaProp']) -> bool: ...
    def xorOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...

class SigmaDsl:
    @staticmethod
    def $init$($this: 'SigmaDsl') -> None: ...
    class AnyValue(scalan.Base.Def['SigmaDsl.AnyValue']):
        def tVal(self) -> scalan.Base.Ref[wrappers.scalan.WRTypes.WRType[typing.Any]]: ...
        def value(self) -> scalan.Base.Ref[typing.Any]: ...
    class AnyValueCompanion: ...
    class AvlTree(scalan.Base.Def['SigmaDsl.AvlTree']):
        def contains(self, key: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def digest(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def enabledOperations(self) -> scalan.Base.Ref[typing.Any]: ...
        def get(self, key: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[special.collection.Colls.Coll[typing.Any]]]: ...
        def getMany(self, keys: scalan.Base.Ref[special.collection.Colls.Coll[special.collection.Colls.Coll[typing.Any]]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[wrappers.scala.WOptions.WOption[special.collection.Colls.Coll[typing.Any]]]]: ...
        def insert(self, operations: scalan.Base.Ref[special.collection.Colls.Coll[scala.Tuple2[special.collection.Colls.Coll[typing.Any], special.collection.Colls.Coll[typing.Any]]]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption['SigmaDsl.AvlTree']]: ...
        def isInsertAllowed(self) -> scalan.Base.Ref[typing.Any]: ...
        def isRemoveAllowed(self) -> scalan.Base.Ref[typing.Any]: ...
        def isUpdateAllowed(self) -> scalan.Base.Ref[typing.Any]: ...
        def keyLength(self) -> scalan.Base.Ref[typing.Any]: ...
        def remove(self, operations: scalan.Base.Ref[special.collection.Colls.Coll[special.collection.Colls.Coll[typing.Any]]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption['SigmaDsl.AvlTree']]: ...
        def update(self, operations: scalan.Base.Ref[special.collection.Colls.Coll[scala.Tuple2[special.collection.Colls.Coll[typing.Any], special.collection.Colls.Coll[typing.Any]]]], proof: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption['SigmaDsl.AvlTree']]: ...
        def updateDigest(self, newDigest: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.AvlTree']: ...
        def updateOperations(self, newOperations: scalan.Base.Ref[typing.Any]) -> scalan.Base.Ref['SigmaDsl.AvlTree']: ...
        def valueLengthOpt(self) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[typing.Any]]: ...
    class AvlTreeCompanion: ...
    class BigInt(scalan.Base.Def['SigmaDsl.BigInt']):
        def add(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def compareTo(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref[typing.Any]: ...
        def divide(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def inverseModQ(self) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def max(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def min(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def minusModQ(self, other: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def mod(self, m: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def modQ(self) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def multModQ(self, other: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def multiply(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def negate(self) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def plusModQ(self, other: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def remainder(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def signum(self) -> scalan.Base.Ref[typing.Any]: ...
        def subtract(self, that: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def toAbs(self) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def toBits(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def toByte(self) -> scalan.Base.Ref[typing.Any]: ...
        def toBytes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def toInt(self) -> scalan.Base.Ref[typing.Any]: ...
        def toLong(self) -> scalan.Base.Ref[typing.Any]: ...
        def toShort(self) -> scalan.Base.Ref[typing.Any]: ...
    class BigIntCompanion: ...
    class Box(scalan.Base.Def['SigmaDsl.Box']):
        @staticmethod
        def $init$($this: 'SigmaDsl.Box') -> None: ...
        _R0__T = typing.TypeVar('_R0__T')  # <T>
        def R0(self, cT: scalan.TypeDescs.Elem[_R0__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R0__T]]: ...
        _R1__T = typing.TypeVar('_R1__T')  # <T>
        def R1(self, cT: scalan.TypeDescs.Elem[_R1__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R1__T]]: ...
        _R2__T = typing.TypeVar('_R2__T')  # <T>
        def R2(self, cT: scalan.TypeDescs.Elem[_R2__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R2__T]]: ...
        _R3__T = typing.TypeVar('_R3__T')  # <T>
        def R3(self, cT: scalan.TypeDescs.Elem[_R3__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R3__T]]: ...
        _R4__T = typing.TypeVar('_R4__T')  # <T>
        def R4(self, cT: scalan.TypeDescs.Elem[_R4__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R4__T]]: ...
        _R5__T = typing.TypeVar('_R5__T')  # <T>
        def R5(self, cT: scalan.TypeDescs.Elem[_R5__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R5__T]]: ...
        _R6__T = typing.TypeVar('_R6__T')  # <T>
        def R6(self, cT: scalan.TypeDescs.Elem[_R6__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R6__T]]: ...
        _R7__T = typing.TypeVar('_R7__T')  # <T>
        def R7(self, cT: scalan.TypeDescs.Elem[_R7__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R7__T]]: ...
        _R8__T = typing.TypeVar('_R8__T')  # <T>
        def R8(self, cT: scalan.TypeDescs.Elem[_R8__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R8__T]]: ...
        _R9__T = typing.TypeVar('_R9__T')  # <T>
        def R9(self, cT: scalan.TypeDescs.Elem[_R9__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_R9__T]]: ...
        def bytes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def bytesWithoutRef(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def creationInfo(self) -> scalan.Base.Ref[scala.Tuple2[typing.Any, special.collection.Colls.Coll[typing.Any]]]: ...
        _executeFromRegister__T = typing.TypeVar('_executeFromRegister__T')  # <T>
        def executeFromRegister(self, regId: scalan.Base.Ref[typing.Any], cT: scalan.TypeDescs.Elem[_executeFromRegister__T]) -> scalan.Base.Ref[_executeFromRegister__T]: ...
        _getReg__T = typing.TypeVar('_getReg__T')  # <T>
        def getReg(self, i: scalan.Base.Ref[typing.Any], cT: scalan.TypeDescs.Elem[_getReg__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_getReg__T]]: ...
        def id(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def propositionBytes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def registers(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.AnyValue']]: ...
        def tokens(self) -> scalan.Base.Ref[special.collection.Colls.Coll[scala.Tuple2[special.collection.Colls.Coll[typing.Any], typing.Any]]]: ...
        def value(self) -> scalan.Base.Ref[typing.Any]: ...
    class BoxCompanion: ...
    class Context(scalan.Base.Def['SigmaDsl.Context']):
        def HEIGHT(self) -> scalan.Base.Ref[typing.Any]: ...
        def INPUTS(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.Box']]: ...
        def LastBlockUtxoRootHash(self) -> scalan.Base.Ref['SigmaDsl.AvlTree']: ...
        def OUTPUTS(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.Box']]: ...
        def SELF(self) -> scalan.Base.Ref['SigmaDsl.Box']: ...
        def builder(self) -> scalan.Base.Ref['SigmaDsl.SigmaDslBuilder']: ...
        def dataInputs(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.Box']]: ...
        _getVar__T = typing.TypeVar('_getVar__T')  # <T>
        def getVar(self, id: scalan.Base.Ref[typing.Any], cT: scalan.TypeDescs.Elem[_getVar__T]) -> scalan.Base.Ref[wrappers.scala.WOptions.WOption[_getVar__T]]: ...
        def headers(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.Header']]: ...
        def minerPubKey(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def preHeader(self) -> scalan.Base.Ref['SigmaDsl.PreHeader']: ...
        def selfBoxIndex(self) -> scalan.Base.Ref[typing.Any]: ...
        def vars(self) -> scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.AnyValue']]: ...
    class ContextCompanion: ...
    class CostModel(scalan.Base.Def['SigmaDsl.CostModel']):
        def AccessAvlTree(self) -> scalan.Base.Ref[typing.Any]: ...
        def AccessBox(self) -> scalan.Base.Ref[typing.Any]: ...
        def AccessKiloByteOfData(self) -> scalan.Base.Ref[typing.Any]: ...
        def CollectionConst(self) -> scalan.Base.Ref[typing.Any]: ...
        def DeserializeRegister(self) -> scalan.Base.Ref[typing.Any]: ...
        def DeserializeVar(self) -> scalan.Base.Ref[typing.Any]: ...
        def GetRegister(self) -> scalan.Base.Ref[typing.Any]: ...
        def GetVar(self) -> scalan.Base.Ref[typing.Any]: ...
        def PubKeySize(self) -> scalan.Base.Ref[typing.Any]: ...
        def SelectField(self) -> scalan.Base.Ref[typing.Any]: ...
    class CostModelCompanion: ...
    class GroupElement(scalan.Base.Def['SigmaDsl.GroupElement']):
        def exp(self, k: scalan.Base.Ref['SigmaDsl.BigInt']) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def getEncoded(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def isInfinity(self) -> scalan.Base.Ref[typing.Any]: ...
        def multiply(self, that: scalan.Base.Ref['SigmaDsl.GroupElement']) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def negate(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
    class GroupElementCompanion: ...
    class Header(scalan.Base.Def['SigmaDsl.Header']):
        def ADProofsRoot(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def extensionRoot(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def height(self) -> scalan.Base.Ref[typing.Any]: ...
        def id(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def minerPk(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def nBits(self) -> scalan.Base.Ref[typing.Any]: ...
        def parentId(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def powDistance(self) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def powNonce(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def powOnetimePk(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def stateRoot(self) -> scalan.Base.Ref['SigmaDsl.AvlTree']: ...
        def timestamp(self) -> scalan.Base.Ref[typing.Any]: ...
        def transactionsRoot(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def version(self) -> scalan.Base.Ref[typing.Any]: ...
        def votes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
    class HeaderCompanion: ...
    class PreHeader(scalan.Base.Def['SigmaDsl.PreHeader']):
        def height(self) -> scalan.Base.Ref[typing.Any]: ...
        def minerPk(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def nBits(self) -> scalan.Base.Ref[typing.Any]: ...
        def parentId(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def timestamp(self) -> scalan.Base.Ref[typing.Any]: ...
        def version(self) -> scalan.Base.Ref[typing.Any]: ...
        def votes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
    class PreHeaderCompanion: ...
    class SigmaContract(scalan.Base.Def['SigmaDsl.SigmaContract']):
        @staticmethod
        def $init$($this: 'SigmaDsl.SigmaContract') -> None: ...
        _Collection__T = typing.TypeVar('_Collection__T')  # <T>
        def Collection(self, items: scala.collection.Seq[scalan.Base.Ref[_Collection__T]], cT: scalan.TypeDescs.Elem[_Collection__T]) -> scalan.Base.Ref[special.collection.Colls.Coll[_Collection__T]]: ...
        def allOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def allZK(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def anyOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def anyZK(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def atLeast(self, bound: scalan.Base.Ref[typing.Any], props: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def blake2b256(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def builder(self) -> scalan.Base.Ref['SigmaDsl.SigmaDslBuilder']: ...
        def byteArrayToBigInt(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def byteArrayToLong(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def decodePoint(self, encoded: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def groupGenerator(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def longToByteArray(self, l: scalan.Base.Ref[typing.Any]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def proveDHTuple(self, g: scalan.Base.Ref['SigmaDsl.GroupElement'], h: scalan.Base.Ref['SigmaDsl.GroupElement'], u: scalan.Base.Ref['SigmaDsl.GroupElement'], v: scalan.Base.Ref['SigmaDsl.GroupElement']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def proveDlog(self, g: scalan.Base.Ref['SigmaDsl.GroupElement']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def sha256(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def sigmaProp(self, b: scalan.Base.Ref[typing.Any]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        _substConstants__T = typing.TypeVar('_substConstants__T')  # <T>
        def substConstants(self, scriptBytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], positions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], newValues: scalan.Base.Ref[special.collection.Colls.Coll[_substConstants__T]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def verifyZK(self, cond: scalan.Base.Ref[scalan.primitives.Thunks.Thunk['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref[typing.Any]: ...
        def xorOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
    class SigmaContractCompanion: ...
    class SigmaDslBuilder(scalan.Base.Def['SigmaDsl.SigmaDslBuilder']):
        def Colls(self) -> scalan.Base.Ref[special.collection.Colls.CollBuilder]: ...
        def CostModel(self) -> scalan.Base.Ref['SigmaDsl.CostModel']: ...
        def Costing(self) -> scalan.Base.Ref[special.collection.Costs.CostedBuilder]: ...
        def Monoids(self) -> scalan.Base.Ref[special.collection.Monoids.MonoidBuilder]: ...
        def allOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def allZK(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def anyOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def anyZK(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def atLeast(self, bound: scalan.Base.Ref[typing.Any], props: scalan.Base.Ref[special.collection.Colls.Coll['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def avlTree(self, operationFlags: scalan.Base.Ref[typing.Any], digest: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], keyLength: scalan.Base.Ref[typing.Any], valueLengthOpt: scalan.Base.Ref[wrappers.scala.WOptions.WOption[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.AvlTree']: ...
        def blake2b256(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def byteArrayToBigInt(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.BigInt']: ...
        def byteArrayToLong(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
        def decodePoint(self, encoded: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def groupGenerator(self) -> scalan.Base.Ref['SigmaDsl.GroupElement']: ...
        def longToByteArray(self, l: scalan.Base.Ref[typing.Any]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def proveDHTuple(self, g: scalan.Base.Ref['SigmaDsl.GroupElement'], h: scalan.Base.Ref['SigmaDsl.GroupElement'], u: scalan.Base.Ref['SigmaDsl.GroupElement'], v: scalan.Base.Ref['SigmaDsl.GroupElement']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def proveDlog(self, g: scalan.Base.Ref['SigmaDsl.GroupElement']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def sha256(self, bytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def sigmaProp(self, b: scalan.Base.Ref[typing.Any]) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        _substConstants__T = typing.TypeVar('_substConstants__T')  # <T>
        def substConstants(self, scriptBytes: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], positions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], newValues: scalan.Base.Ref[special.collection.Colls.Coll[_substConstants__T]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def verifyZK(self, cond: scalan.Base.Ref[scalan.primitives.Thunks.Thunk['SigmaDsl.SigmaProp']]) -> scalan.Base.Ref[typing.Any]: ...
        def xor(self, l: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]], r: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
        def xorOf(self, conditions: scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]) -> scalan.Base.Ref[typing.Any]: ...
    class SigmaDslBuilderCompanion: ...
    class SigmaProp(scalan.Base.Def['SigmaDsl.SigmaProp']):
        @typing.overload
        def $amp$amp(self, other: scalan.Base.Ref['SigmaDsl.SigmaProp']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        @typing.overload
        def $amp$amp(self, other: scalan.Base.Ref[typing.Any], o: scalan.OverloadHack.Overloaded1) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        @typing.overload
        def $bar$bar(self, other: scalan.Base.Ref['SigmaDsl.SigmaProp']) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        @typing.overload
        def $bar$bar(self, other: scalan.Base.Ref[typing.Any], o: scalan.OverloadHack.Overloaded1) -> scalan.Base.Ref['SigmaDsl.SigmaProp']: ...
        def isValid(self) -> scalan.Base.Ref[typing.Any]: ...
        def propBytes(self) -> scalan.Base.Ref[special.collection.Colls.Coll[typing.Any]]: ...
    class SigmaPropCompanion: ...

class SigmaDslBuilder:
    def BigInt(self, n: java.math.BigInteger) -> BigInt: ...
    def Colls(self) -> special.collection.CollBuilder: ...
    def CostModel(self) -> CostModel: ...
    def Costing(self) -> special.collection.CostedBuilder: ...
    def Monoids(self) -> special.collection.MonoidBuilder: ...
    def allOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def allZK(self, conditions: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def anyOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def anyZK(self, conditions: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def atLeast(self, bound: int, props: special.collection.Coll['SigmaProp']) -> 'SigmaProp': ...
    def avlTree(self, operationFlags: int, digest: special.collection.Coll[typing.Any], keyLength: int, valueLengthOpt: scala.Option[typing.Any]) -> AvlTree: ...
    def blake2b256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def byteArrayToBigInt(self, bytes: special.collection.Coll[typing.Any]) -> BigInt: ...
    def byteArrayToLong(self, bytes: special.collection.Coll[typing.Any]) -> int: ...
    def decodePoint(self, encoded: special.collection.Coll[typing.Any]) -> GroupElement: ...
    def groupGenerator(self) -> GroupElement: ...
    def longToByteArray(self, l: int) -> special.collection.Coll[typing.Any]: ...
    def proveDHTuple(self, g: GroupElement, h: GroupElement, u: GroupElement, v: GroupElement) -> 'SigmaProp': ...
    def proveDlog(self, g: GroupElement) -> 'SigmaProp': ...
    def sha256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def sigmaProp(self, b: bool) -> 'SigmaProp': ...
    _substConstants__T = typing.TypeVar('_substConstants__T')  # <T>
    def substConstants(self, scriptBytes: special.collection.Coll[typing.Any], positions: special.collection.Coll[typing.Any], newValues: special.collection.Coll[_substConstants__T]) -> special.collection.Coll[typing.Any]: ...
    def toBigInteger(self, n: BigInt) -> java.math.BigInteger: ...
    def verifyZK(self, cond: scala.Function0['SigmaProp']) -> bool: ...
    def xor(self, l: special.collection.Coll[typing.Any], r: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def xorOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...

class SigmaDslModule(special.sigma.impl.SigmaDslDefs): ...

class SigmaProp:
    @typing.overload
    def $amp$amp(self, other: bool) -> 'SigmaProp': ...
    @typing.overload
    def $amp$amp(self, other: 'SigmaProp') -> 'SigmaProp': ...
    @typing.overload
    def $bar$bar(self, other: bool) -> 'SigmaProp': ...
    @typing.overload
    def $bar$bar(self, other: 'SigmaProp') -> 'SigmaProp': ...
    def isValid(self) -> bool: ...
    def propBytes(self) -> special.collection.Coll[typing.Any]: ...

class SizeAnyValue(special.collection.Size[AnyValue]):
    def tVal(self) -> scalan.RType[typing.Any]: ...
    def valueSize(self) -> special.collection.Size[typing.Any]: ...

class SizeBox(special.collection.Size[Box]):
    def bytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    def bytesWithoutRef(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    _getReg__T = typing.TypeVar('_getReg__T')  # <T>
    def getReg(self, id: int, tT: scalan.RType[_getReg__T]) -> special.collection.Size[scala.Option[_getReg__T]]: ...
    def propositionBytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    def registers(self) -> special.collection.Size[special.collection.Coll[scala.Option[AnyValue]]]: ...
    def tokens(self) -> special.collection.Size[special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]]: ...

class SizeBuilder:
    def mkSizeAnyValue(self, tVal: scalan.RType[typing.Any], valueSize: special.collection.Size[typing.Any]) -> SizeAnyValue: ...
    def mkSizeBox(self, propositionBytes: special.collection.Size[special.collection.Coll[typing.Any]], bytes: special.collection.Size[special.collection.Coll[typing.Any]], bytesWithoutRef: special.collection.Size[special.collection.Coll[typing.Any]], registers: special.collection.Size[special.collection.Coll[scala.Option[AnyValue]]], tokens: special.collection.Size[special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]]) -> SizeBox: ...
    def mkSizeContext(self, outputs: special.collection.Size[special.collection.Coll[Box]], inputs: special.collection.Size[special.collection.Coll[Box]], dataInputs: special.collection.Size[special.collection.Coll[Box]], selfBox: special.collection.Size[Box], lastBlockUtxoRootHash: special.collection.Size[AvlTree], headers: special.collection.Size[special.collection.Coll[Header]], preHeader: special.collection.Size[PreHeader], vars: special.collection.Coll[special.collection.Size[AnyValue]]) -> 'SizeContext': ...

class SizeContext(special.collection.Size[Context]):
    def dataInputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    _getVar__T = typing.TypeVar('_getVar__T')  # <T>
    def getVar(self, id: int, tT: scalan.RType[_getVar__T]) -> special.collection.Size[scala.Option[_getVar__T]]: ...
    def headers(self) -> special.collection.Size[special.collection.Coll[Header]]: ...
    def inputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    def lastBlockUtxoRootHash(self) -> special.collection.Size[AvlTree]: ...
    def outputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    def preHeader(self) -> special.collection.Size[PreHeader]: ...
    def selfBox(self) -> special.collection.Size[Box]: ...

class SizeSigmaProp(special.collection.Size[SigmaProp]):
    def propBytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...

class package:
    @staticmethod
    def AnyValueRType() -> scalan.RType[AnyValue]: ...
    @staticmethod
    def AvlTreeRType() -> scalan.RType[AvlTree]: ...
    @staticmethod
    def BigIntRType() -> scalan.RType[BigInt]: ...
    @staticmethod
    def BigIntegerRType() -> scalan.RType[java.math.BigInteger]: ...
    @staticmethod
    def BoxRType() -> scalan.RType[Box]: ...
    @staticmethod
    def ContextRType() -> scalan.RType[Context]: ...
    @staticmethod
    def CostModelRType() -> scalan.RType[CostModel]: ...
    @staticmethod
    def ECPointRType() -> scalan.RType[org.bouncycastle.math.ec.ECPoint]: ...
    @staticmethod
    def GroupElementRType() -> scalan.RType[GroupElement]: ...
    @staticmethod
    def HeaderRType() -> scalan.RType[Header]: ...
    @staticmethod
    def PreHeaderRType() -> scalan.RType[PreHeader]: ...
    @staticmethod
    def SigmaContractRType() -> scalan.RType[SigmaContract]: ...
    @staticmethod
    def SigmaDslBuilderRType() -> scalan.RType[SigmaDslBuilder]: ...
    @staticmethod
    def SigmaPropRType() -> scalan.RType[SigmaProp]: ...
    @staticmethod
    def SizeAnyValueRType() -> scalan.RType[SizeAnyValue]: ...
    @staticmethod
    def SizeBoxRType() -> scalan.RType[SizeBox]: ...
    @staticmethod
    def SizeBuilderRType() -> scalan.RType[SizeBuilder]: ...
    @staticmethod
    def SizeContextRType() -> scalan.RType[SizeContext]: ...
    @staticmethod
    def SizeSigmaPropRType() -> scalan.RType[SizeSigmaProp]: ...

class CSizeAnyValue(SizeAnyValue):
    def __init__(self, tVal: scalan.RType[typing.Any], valueSize: special.collection.Size[typing.Any]): ...
    def dataSize(self) -> int: ...
    def tVal(self) -> scalan.RType[typing.Any]: ...
    def valueSize(self) -> special.collection.Size[typing.Any]: ...

class CSizeBox(SizeBox):
    def __init__(self, propositionBytes: special.collection.Size[special.collection.Coll[typing.Any]], bytes: special.collection.Size[special.collection.Coll[typing.Any]], bytesWithoutRef: special.collection.Size[special.collection.Coll[typing.Any]], registers: special.collection.Size[special.collection.Coll[scala.Option[AnyValue]]], tokens: special.collection.Size[special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]]): ...
    def bytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    def bytesWithoutRef(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    def dataSize(self) -> int: ...
    _getReg__T = typing.TypeVar('_getReg__T')  # <T>
    def getReg(self, id: int, tT: scalan.RType[_getReg__T]) -> special.collection.Size[scala.Option[_getReg__T]]: ...
    def propositionBytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...
    def registers(self) -> special.collection.Size[special.collection.Coll[scala.Option[AnyValue]]]: ...
    def tokens(self) -> special.collection.Size[special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]]: ...

class CSizeBuilder(SizeBuilder):
    def __init__(self): ...
    def mkSizeAnyValue(self, tVal: scalan.RType[typing.Any], valueSize: special.collection.Size[typing.Any]) -> SizeAnyValue: ...
    def mkSizeBox(self, propositionBytes: special.collection.Size[special.collection.Coll[typing.Any]], bytes: special.collection.Size[special.collection.Coll[typing.Any]], bytesWithoutRef: special.collection.Size[special.collection.Coll[typing.Any]], registers: special.collection.Size[special.collection.Coll[scala.Option[AnyValue]]], tokens: special.collection.Size[special.collection.Coll[scala.Tuple2[special.collection.Coll[typing.Any], typing.Any]]]) -> SizeBox: ...
    def mkSizeContext(self, outputs: special.collection.Size[special.collection.Coll[Box]], inputs: special.collection.Size[special.collection.Coll[Box]], dataInputs: special.collection.Size[special.collection.Coll[Box]], selfBox: special.collection.Size[Box], lastBlockUtxoRootHash: special.collection.Size[AvlTree], headers: special.collection.Size[special.collection.Coll[Header]], preHeader: special.collection.Size[PreHeader], vars: special.collection.Coll[special.collection.Size[AnyValue]]) -> SizeContext: ...

class CSizeContext(SizeContext):
    def __init__(self, outputs: special.collection.Size[special.collection.Coll[Box]], inputs: special.collection.Size[special.collection.Coll[Box]], dataInputs: special.collection.Size[special.collection.Coll[Box]], selfBox: special.collection.Size[Box], lastBlockUtxoRootHash: special.collection.Size[AvlTree], headers: special.collection.Size[special.collection.Coll[Header]], preHeader: special.collection.Size[PreHeader], vars: special.collection.Coll[special.collection.Size[AnyValue]]): ...
    def dataInputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    def dataSize(self) -> int: ...
    _getVar__T = typing.TypeVar('_getVar__T')  # <T>
    def getVar(self, id: int, tT: scalan.RType[_getVar__T]) -> special.collection.Size[scala.Option[_getVar__T]]: ...
    def headers(self) -> special.collection.Size[special.collection.Coll[Header]]: ...
    def inputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    def lastBlockUtxoRootHash(self) -> special.collection.Size[AvlTree]: ...
    def outputs(self) -> special.collection.Size[special.collection.Coll[Box]]: ...
    def preHeader(self) -> special.collection.Size[PreHeader]: ...
    def selfBox(self) -> special.collection.Size[Box]: ...
    def vars(self) -> special.collection.Coll[special.collection.Size[AnyValue]]: ...

class CSizeSigmaProp(SizeSigmaProp):
    def __init__(self, propBytes: special.collection.Size[special.collection.Coll[typing.Any]]): ...
    def dataSize(self) -> int: ...
    def propBytes(self) -> special.collection.Size[special.collection.Coll[typing.Any]]: ...

class MockProveDlog(SigmaProp, scala.Product, scala.Serializable):
    def __init__(self, isValid: bool, propBytes: special.collection.Coll[typing.Any]): ...
    @typing.overload
    def $amp$amp(self, other: bool) -> SigmaProp: ...
    @typing.overload
    def $amp$amp(self, other: SigmaProp) -> SigmaProp: ...
    @typing.overload
    def $bar$bar(self, other: bool) -> SigmaProp: ...
    @typing.overload
    def $bar$bar(self, other: SigmaProp) -> SigmaProp: ...
    @staticmethod
    def apply(isValid: bool, propBytes: special.collection.Coll[typing.Any]) -> 'MockProveDlog': ...
    def builder(self) -> 'TestSigmaDslBuilder': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def copy(self, isValid: bool, propBytes: special.collection.Coll[typing.Any]) -> 'MockProveDlog': ...
    def copy$default$1(self) -> bool: ...
    def copy$default$2(self) -> special.collection.Coll[typing.Any]: ...
    @staticmethod
    def curried() -> scala.Function1[typing.Any, scala.Function1[special.collection.Coll[typing.Any], 'MockProveDlog']]: ...
    def curve(self) -> org.bouncycastle.asn1.x9.X9ECParameters: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def isValid(self) -> bool: ...
    def isValid_$eq(self, x$1: bool) -> None: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def propBytes(self) -> special.collection.Coll[typing.Any]: ...
    def setValid(self, v: bool) -> None: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def tupled() -> scala.Function1[scala.Tuple2[typing.Any, special.collection.Coll[typing.Any]], 'MockProveDlog']: ...
    @staticmethod
    def unapply(x$0: 'MockProveDlog') -> scala.Option[scala.Tuple2[typing.Any, special.collection.Coll[typing.Any]]]: ...
    def value(self) -> org.bouncycastle.math.ec.ECPoint: ...

class MockSigma(SigmaProp, scala.Product, scala.Serializable):
    def __init__(self, _isValid: bool): ...
    @typing.overload
    def $amp$amp(self, other: bool) -> SigmaProp: ...
    @typing.overload
    def $amp$amp(self, other: SigmaProp) -> SigmaProp: ...
    @typing.overload
    def $bar$bar(self, other: bool) -> SigmaProp: ...
    @typing.overload
    def $bar$bar(self, other: SigmaProp) -> SigmaProp: ...
    def _isValid(self) -> bool: ...
    _andThen__A = typing.TypeVar('_andThen__A')  # <A>
    @staticmethod
    def andThen(g: scala.Function1['MockSigma', _andThen__A]) -> scala.Function1[typing.Any, _andThen__A]: ...
    @staticmethod
    def apply(_isValid: bool) -> 'MockSigma': ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    @staticmethod
    def compose(g: scala.Function1[_compose__A, typing.Any]) -> scala.Function1[_compose__A, 'MockSigma']: ...
    def copy(self, _isValid: bool) -> 'MockSigma': ...
    def copy$default$1(self) -> bool: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def isValid(self) -> bool: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def propBytes(self) -> special.collection.Coll[typing.Any]: ...
    def toString(self) -> java.lang.String: ...
    @staticmethod
    def unapply(x$0: 'MockSigma') -> scala.Option[typing.Any]: ...

class TestBigInt(BigInt):
    def __init__(self, value: java.math.BigInteger): ...
    def $amp(self, that: BigInt) -> BigInt: ...
    def $bar(self, that: BigInt) -> BigInt: ...
    def $div(self, that: BigInt) -> BigInt: ...
    def $minus(self, that: BigInt) -> BigInt: ...
    def $percent(self, m: BigInt) -> BigInt: ...
    def $plus(self, that: BigInt) -> BigInt: ...
    def $times(self, that: BigInt) -> BigInt: ...
    def add(self, that: BigInt) -> BigInt: ...
    def compareTo(self, that: BigInt) -> int: ...
    def divide(self, that: BigInt) -> BigInt: ...
    def dsl(self) -> 'TestSigmaDslBuilder': ...
    def inverseModQ(self) -> BigInt: ...
    def max(self, that: BigInt) -> BigInt: ...
    def min(self, that: BigInt) -> BigInt: ...
    def minusModQ(self, other: BigInt) -> BigInt: ...
    def mod(self, m: BigInt) -> BigInt: ...
    def modQ(self) -> BigInt: ...
    def multModQ(self, other: BigInt) -> BigInt: ...
    def multiply(self, that: BigInt) -> BigInt: ...
    def negate(self) -> BigInt: ...
    def plusModQ(self, other: BigInt) -> BigInt: ...
    def remainder(self, that: BigInt) -> BigInt: ...
    def signum(self) -> int: ...
    def subtract(self, that: BigInt) -> BigInt: ...
    def toAbs(self) -> BigInt: ...
    def toBits(self) -> special.collection.Coll[typing.Any]: ...
    def toByte(self) -> int: ...
    def toBytes(self) -> special.collection.Coll[typing.Any]: ...
    def toInt(self) -> int: ...
    def toLong(self) -> int: ...
    def toShort(self) -> int: ...
    def value(self) -> java.math.BigInteger: ...

class TestGroupElement(GroupElement):
    def __init__(self, value: org.bouncycastle.math.ec.ECPoint): ...
    def dsl(self) -> 'TestSigmaDslBuilder': ...
    def exp(self, k: BigInt) -> GroupElement: ...
    def isInfinity(self) -> bool: ...
    def multiply(self, that: GroupElement) -> GroupElement: ...
    def negate(self) -> GroupElement: ...
    def toString(self) -> java.lang.String: ...
    def value(self) -> org.bouncycastle.math.ec.ECPoint: ...

class TestSigmaDslBuilder(SigmaDslBuilder):
    def __init__(self): ...
    def BigInt(self, n: java.math.BigInteger) -> BigInt: ...
    def Colls(self) -> special.collection.CollBuilder: ...
    def CostModel(self) -> CostModel: ...
    def Costing(self) -> special.collection.CostedBuilder: ...
    def GroupElement(self, p: org.bouncycastle.math.ec.ECPoint) -> GroupElement: ...
    def Monoids(self) -> special.collection.MonoidBuilder: ...
    def __curve__(self) -> org.bouncycastle.asn1.x9.X9ECParameters: ...
    def __g__(self) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    def allOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def allZK(self, props: special.collection.Coll[SigmaProp]) -> SigmaProp: ...
    def anyOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...
    def anyZK(self, props: special.collection.Coll[SigmaProp]) -> SigmaProp: ...
    def atLeast(self, bound: int, props: special.collection.Coll[SigmaProp]) -> SigmaProp: ...
    def avlTree(self, operationFlags: int, digest: special.collection.Coll[typing.Any], keyLength: int, valueLengthOpt: scala.Option[typing.Any]) -> AvlTree: ...
    def blake2b256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def byteArrayToBigInt(self, bytes: special.collection.Coll[typing.Any]) -> BigInt: ...
    def byteArrayToLong(self, bytes: special.collection.Coll[typing.Any]) -> int: ...
    def decodePoint(self, encoded: special.collection.Coll[typing.Any]) -> GroupElement: ...
    def groupGenerator(self) -> GroupElement: ...
    def longToByteArray(self, l: int) -> special.collection.Coll[typing.Any]: ...
    def proveDHTuple(self, g: GroupElement, h: GroupElement, u: GroupElement, v: GroupElement) -> SigmaProp: ...
    def proveDlog(self, g: GroupElement) -> SigmaProp: ...
    def sha256(self, bytes: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def sigmaProp(self, b: bool) -> SigmaProp: ...
    _substConstants__T = typing.TypeVar('_substConstants__T')  # <T>
    def substConstants(self, scriptBytes: special.collection.Coll[typing.Any], positions: special.collection.Coll[typing.Any], newValues: special.collection.Coll[_substConstants__T]) -> special.collection.Coll[typing.Any]: ...
    def toBigInteger(self, n: BigInt) -> java.math.BigInteger: ...
    def toECPoint(self, ge: GroupElement) -> org.bouncycastle.math.ec.ECPoint: ...
    def verifyZK(self, proof: scala.Function0[SigmaProp]) -> bool: ...
    def xor(self, l: special.collection.Coll[typing.Any], r: special.collection.Coll[typing.Any]) -> special.collection.Coll[typing.Any]: ...
    def xorOf(self, conditions: special.collection.Coll[typing.Any]) -> bool: ...

_TestValue__A = typing.TypeVar('_TestValue__A')  # <A>
class TestValue(AnyValue, scala.Product, scala.Serializable, typing.Generic[_TestValue__A]):
    def __init__(self, value: _TestValue__A, tVal: scalan.RType[typing.Any]): ...
    _apply_0__A = typing.TypeVar('_apply_0__A')  # <A>
    _apply_1__A = typing.TypeVar('_apply_1__A')  # <A>
    @typing.overload
    @staticmethod
    def apply(value: _apply_0__A, tVal: scalan.RType[typing.Any]) -> 'TestValue'[_apply_0__A]: ...
    @typing.overload
    @staticmethod
    def apply(value: _apply_1__A, t: scalan.RType[_apply_1__A], o: scalan.OverloadHack.Overloaded1) -> 'TestValue'[_apply_1__A]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    _copy__A = typing.TypeVar('_copy__A')  # <A>
    def copy(self, value: typing.Any, tVal: scalan.RType[typing.Any]) -> 'TestValue'[typing.Any]: ...
    _copy$default$1__A = typing.TypeVar('_copy$default$1__A')  # <A>
    def copy$default$1(self) -> typing.Any: ...
    _copy$default$2__A = typing.TypeVar('_copy$default$2__A')  # <A>
    def copy$default$2(self) -> scalan.RType[typing.Any]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def hashCode(self) -> int: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def tA(self) -> scalan.RType[_TestValue__A]: ...
    def tVal(self) -> scalan.RType[typing.Any]: ...
    def toString(self) -> java.lang.String: ...
    _unapply__A = typing.TypeVar('_unapply__A')  # <A>
    @staticmethod
    def unapply(x$0: 'TestValue'[_unapply__A]) -> scala.Option[scala.Tuple2[_unapply__A, scalan.RType[typing.Any]]]: ...
    def value(self) -> _TestValue__A: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("special.sigma")``.

    AnyValue: typing.Type[AnyValue]
    AvlTree: typing.Type[AvlTree]
    AvlTreeVerifier: typing.Type[AvlTreeVerifier]
    BigInt: typing.Type[BigInt]
    Box: typing.Type[Box]
    CSizeAnyValue: typing.Type[CSizeAnyValue]
    CSizeBox: typing.Type[CSizeBox]
    CSizeBuilder: typing.Type[CSizeBuilder]
    CSizeContext: typing.Type[CSizeContext]
    CSizeSigmaProp: typing.Type[CSizeSigmaProp]
    Context: typing.Type[Context]
    CostModel: typing.Type[CostModel]
    CostedObjects: typing.Type[CostedObjects]
    CostedObjectsModule: typing.Type[CostedObjectsModule]
    DslSyntaxExtensions: typing.Type[DslSyntaxExtensions]
    Extensions: typing.Type[Extensions]
    GroupElement: typing.Type[GroupElement]
    Header: typing.Type[Header]
    InvalidType: typing.Type[InvalidType]
    MockProveDlog: typing.Type[MockProveDlog]
    MockSigma: typing.Type[MockSigma]
    PreHeader: typing.Type[PreHeader]
    SigmaContract: typing.Type[SigmaContract]
    SigmaDsl: typing.Type[SigmaDsl]
    SigmaDslBuilder: typing.Type[SigmaDslBuilder]
    SigmaDslModule: typing.Type[SigmaDslModule]
    SigmaProp: typing.Type[SigmaProp]
    SizeAnyValue: typing.Type[SizeAnyValue]
    SizeBox: typing.Type[SizeBox]
    SizeBuilder: typing.Type[SizeBuilder]
    SizeContext: typing.Type[SizeContext]
    SizeSigmaProp: typing.Type[SizeSigmaProp]
    TestBigInt: typing.Type[TestBigInt]
    TestGroupElement: typing.Type[TestGroupElement]
    TestSigmaDslBuilder: typing.Type[TestSigmaDslBuilder]
    TestValue: typing.Type[TestValue]
    package: typing.Type[package]
    wrappers: special.sigma.wrappers.__module_protocol__



import java
import java.io
import java.lang
import java.math
import java.net
import java.security
import java.util
import javax.security.auth.x500
import typing



class CRL:
    def getType(self) -> str: ...
    def isRevoked(self, certificate: 'Certificate') -> bool: ...
    def toString(self) -> str: ...

class CRLException(java.security.GeneralSecurityException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CRLReason(java.lang.Enum['CRLReason']):
    UNSPECIFIED: typing.ClassVar['CRLReason'] = ...
    KEY_COMPROMISE: typing.ClassVar['CRLReason'] = ...
    CA_COMPROMISE: typing.ClassVar['CRLReason'] = ...
    AFFILIATION_CHANGED: typing.ClassVar['CRLReason'] = ...
    SUPERSEDED: typing.ClassVar['CRLReason'] = ...
    CESSATION_OF_OPERATION: typing.ClassVar['CRLReason'] = ...
    CERTIFICATE_HOLD: typing.ClassVar['CRLReason'] = ...
    UNUSED: typing.ClassVar['CRLReason'] = ...
    REMOVE_FROM_CRL: typing.ClassVar['CRLReason'] = ...
    PRIVILEGE_WITHDRAWN: typing.ClassVar['CRLReason'] = ...
    AA_COMPROMISE: typing.ClassVar['CRLReason'] = ...
    _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
    @typing.overload
    @staticmethod
    def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
    @typing.overload
    @staticmethod
    def valueOf(string: str) -> 'CRLReason': ...
    @staticmethod
    def values() -> typing.List['CRLReason']: ...

class CRLSelector(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...
    def match(self, cRL: CRL) -> bool: ...

class CertPath(java.io.Serializable):
    def equals(self, object: typing.Any) -> bool: ...
    def getCertificates(self) -> java.util.List['Certificate']: ...
    @typing.overload
    def getEncoded(self) -> typing.List[int]: ...
    @typing.overload
    def getEncoded(self, string: str) -> typing.List[int]: ...
    def getEncodings(self) -> java.util.Iterator[str]: ...
    def getType(self) -> str: ...
    def hashCode(self) -> int: ...
    def toString(self) -> str: ...

class CertPathBuilder:
    def build(self, certPathParameters: 'CertPathParameters') -> 'CertPathBuilderResult': ...
    def getAlgorithm(self) -> str: ...
    @staticmethod
    def getDefaultType() -> str: ...
    @typing.overload
    @staticmethod
    def getInstance(string: str) -> 'CertPathBuilder': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, string2: str) -> 'CertPathBuilder': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, provider: java.security.Provider) -> 'CertPathBuilder': ...
    def getProvider(self) -> java.security.Provider: ...
    def getRevocationChecker(self) -> 'CertPathChecker': ...

class CertPathBuilderException(java.security.GeneralSecurityException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CertPathBuilderResult(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...
    def getCertPath(self) -> CertPath: ...

class CertPathBuilderSpi:
    def __init__(self): ...
    def engineBuild(self, certPathParameters: 'CertPathParameters') -> CertPathBuilderResult: ...
    def engineGetRevocationChecker(self) -> 'CertPathChecker': ...

class CertPathChecker:
    def check(self, certificate: 'Certificate') -> None: ...
    def init(self, boolean: bool) -> None: ...
    def isForwardCheckingSupported(self) -> bool: ...

class CertPathParameters(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...

class CertPathValidator:
    def getAlgorithm(self) -> str: ...
    @staticmethod
    def getDefaultType() -> str: ...
    @typing.overload
    @staticmethod
    def getInstance(string: str) -> 'CertPathValidator': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, string2: str) -> 'CertPathValidator': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, provider: java.security.Provider) -> 'CertPathValidator': ...
    def getProvider(self) -> java.security.Provider: ...
    def getRevocationChecker(self) -> CertPathChecker: ...
    def validate(self, certPath: CertPath, certPathParameters: CertPathParameters) -> 'CertPathValidatorResult': ...

class CertPathValidatorResult(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...

class CertPathValidatorSpi:
    def __init__(self): ...
    def engineGetRevocationChecker(self) -> CertPathChecker: ...
    def engineValidate(self, certPath: CertPath, certPathParameters: CertPathParameters) -> CertPathValidatorResult: ...

class CertSelector(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...
    def match(self, certificate: 'Certificate') -> bool: ...

class CertStore:
    def getCRLs(self, cRLSelector: CRLSelector) -> java.util.Collection[CRL]: ...
    def getCertStoreParameters(self) -> 'CertStoreParameters': ...
    def getCertificates(self, certSelector: CertSelector) -> java.util.Collection['Certificate']: ...
    @staticmethod
    def getDefaultType() -> str: ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, certStoreParameters: 'CertStoreParameters') -> 'CertStore': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, certStoreParameters: 'CertStoreParameters', string2: str) -> 'CertStore': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, certStoreParameters: 'CertStoreParameters', provider: java.security.Provider) -> 'CertStore': ...
    def getProvider(self) -> java.security.Provider: ...
    def getType(self) -> str: ...

class CertStoreException(java.security.GeneralSecurityException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CertStoreParameters(java.lang.Cloneable):
    def clone(self) -> typing.Any: ...

class CertStoreSpi:
    def __init__(self, certStoreParameters: CertStoreParameters): ...
    def engineGetCRLs(self, cRLSelector: CRLSelector) -> java.util.Collection[CRL]: ...
    def engineGetCertificates(self, certSelector: CertSelector) -> java.util.Collection['Certificate']: ...

class Certificate(java.io.Serializable):
    def equals(self, object: typing.Any) -> bool: ...
    def getEncoded(self) -> typing.List[int]: ...
    def getPublicKey(self) -> java.security.PublicKey: ...
    def getType(self) -> str: ...
    def hashCode(self) -> int: ...
    def toString(self) -> str: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, string: str) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, provider: java.security.Provider) -> None: ...

class CertificateException(java.security.GeneralSecurityException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CertificateFactory:
    def generateCRL(self, inputStream: java.io.InputStream) -> CRL: ...
    def generateCRLs(self, inputStream: java.io.InputStream) -> java.util.Collection[CRL]: ...
    @typing.overload
    def generateCertPath(self, inputStream: java.io.InputStream) -> CertPath: ...
    @typing.overload
    def generateCertPath(self, inputStream: java.io.InputStream, string: str) -> CertPath: ...
    @typing.overload
    def generateCertPath(self, list: java.util.List[Certificate]) -> CertPath: ...
    def generateCertificate(self, inputStream: java.io.InputStream) -> Certificate: ...
    def generateCertificates(self, inputStream: java.io.InputStream) -> java.util.Collection[Certificate]: ...
    def getCertPathEncodings(self) -> java.util.Iterator[str]: ...
    @typing.overload
    @staticmethod
    def getInstance(string: str) -> 'CertificateFactory': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, string2: str) -> 'CertificateFactory': ...
    @typing.overload
    @staticmethod
    def getInstance(string: str, provider: java.security.Provider) -> 'CertificateFactory': ...
    def getProvider(self) -> java.security.Provider: ...
    def getType(self) -> str: ...

class CertificateFactorySpi:
    def __init__(self): ...
    def engineGenerateCRL(self, inputStream: java.io.InputStream) -> CRL: ...
    def engineGenerateCRLs(self, inputStream: java.io.InputStream) -> java.util.Collection[CRL]: ...
    @typing.overload
    def engineGenerateCertPath(self, inputStream: java.io.InputStream) -> CertPath: ...
    @typing.overload
    def engineGenerateCertPath(self, inputStream: java.io.InputStream, string: str) -> CertPath: ...
    @typing.overload
    def engineGenerateCertPath(self, list: java.util.List[Certificate]) -> CertPath: ...
    def engineGenerateCertificate(self, inputStream: java.io.InputStream) -> Certificate: ...
    def engineGenerateCertificates(self, inputStream: java.io.InputStream) -> java.util.Collection[Certificate]: ...
    def engineGetCertPathEncodings(self) -> java.util.Iterator[str]: ...

class Extension:
    def encode(self, outputStream: java.io.OutputStream) -> None: ...
    def getId(self) -> str: ...
    def getValue(self) -> typing.List[int]: ...
    def isCritical(self) -> bool: ...

class PolicyNode:
    def getChildren(self) -> java.util.Iterator['PolicyNode']: ...
    def getDepth(self) -> int: ...
    def getExpectedPolicies(self) -> java.util.Set[str]: ...
    def getParent(self) -> 'PolicyNode': ...
    def getPolicyQualifiers(self) -> java.util.Set['PolicyQualifierInfo']: ...
    def getValidPolicy(self) -> str: ...
    def isCritical(self) -> bool: ...

class PolicyQualifierInfo:
    def __init__(self, byteArray: typing.List[int]): ...
    def getEncoded(self) -> typing.List[int]: ...
    def getPolicyQualifier(self) -> typing.List[int]: ...
    def getPolicyQualifierId(self) -> str: ...
    def toString(self) -> str: ...

class TrustAnchor:
    @typing.overload
    def __init__(self, string: str, publicKey: java.security.PublicKey, byteArray: typing.List[int]): ...
    @typing.overload
    def __init__(self, x509Certificate: 'X509Certificate', byteArray: typing.List[int]): ...
    @typing.overload
    def __init__(self, x500Principal: javax.security.auth.x500.X500Principal, publicKey: java.security.PublicKey, byteArray: typing.List[int]): ...
    def getCA(self) -> javax.security.auth.x500.X500Principal: ...
    def getCAName(self) -> str: ...
    def getCAPublicKey(self) -> java.security.PublicKey: ...
    def getNameConstraints(self) -> typing.List[int]: ...
    def getTrustedCert(self) -> 'X509Certificate': ...
    def toString(self) -> str: ...

class X509Extension:
    def getCriticalExtensionOIDs(self) -> java.util.Set[str]: ...
    def getExtensionValue(self, string: str) -> typing.List[int]: ...
    def getNonCriticalExtensionOIDs(self) -> java.util.Set[str]: ...
    def hasUnsupportedCriticalExtension(self) -> bool: ...

class CertificateEncodingException(CertificateException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CertificateExpiredException(CertificateException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...

class CertificateNotYetValidException(CertificateException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...

class CertificateParsingException(CertificateException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...

class CertificateRevokedException(CertificateException):
    def __init__(self, date: java.util.Date, cRLReason: CRLReason, x500Principal: javax.security.auth.x500.X500Principal, map: typing.Union[java.util.Map[str, Extension], typing.Mapping[str, Extension]]): ...
    def getAuthorityName(self) -> javax.security.auth.x500.X500Principal: ...
    def getExtensions(self) -> java.util.Map[str, Extension]: ...
    def getInvalidityDate(self) -> java.util.Date: ...
    def getMessage(self) -> str: ...
    def getRevocationDate(self) -> java.util.Date: ...
    def getRevocationReason(self) -> CRLReason: ...

class CollectionCertStoreParameters(CertStoreParameters):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, collection: typing.Union[java.util.Collection[typing.Any], typing.Sequence[typing.Any], typing.Set[typing.Any]]): ...
    def clone(self) -> typing.Any: ...
    def getCollection(self) -> java.util.Collection[typing.Any]: ...
    def toString(self) -> str: ...

class LDAPCertStoreParameters(CertStoreParameters):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, int: int): ...
    def clone(self) -> typing.Any: ...
    def getPort(self) -> int: ...
    def getServerName(self) -> str: ...
    def toString(self) -> str: ...

class PKIXCertPathChecker(CertPathChecker, java.lang.Cloneable):
    @typing.overload
    def check(self, certificate: Certificate, collection: typing.Union[java.util.Collection[str], typing.Sequence[str], typing.Set[str]]) -> None: ...
    @typing.overload
    def check(self, certificate: Certificate) -> None: ...
    def clone(self) -> typing.Any: ...
    def getSupportedExtensions(self) -> java.util.Set[str]: ...
    def init(self, boolean: bool) -> None: ...
    def isForwardCheckingSupported(self) -> bool: ...

class PKIXCertPathValidatorResult(CertPathValidatorResult):
    def __init__(self, trustAnchor: TrustAnchor, policyNode: PolicyNode, publicKey: java.security.PublicKey): ...
    def clone(self) -> typing.Any: ...
    def getPolicyTree(self) -> PolicyNode: ...
    def getPublicKey(self) -> java.security.PublicKey: ...
    def getTrustAnchor(self) -> TrustAnchor: ...
    def toString(self) -> str: ...

class PKIXParameters(CertPathParameters):
    @typing.overload
    def __init__(self, keyStore: java.security.KeyStore): ...
    @typing.overload
    def __init__(self, set: java.util.Set[TrustAnchor]): ...
    def addCertPathChecker(self, pKIXCertPathChecker: PKIXCertPathChecker) -> None: ...
    def addCertStore(self, certStore: CertStore) -> None: ...
    def clone(self) -> typing.Any: ...
    def getCertPathCheckers(self) -> java.util.List[PKIXCertPathChecker]: ...
    def getCertStores(self) -> java.util.List[CertStore]: ...
    def getDate(self) -> java.util.Date: ...
    def getInitialPolicies(self) -> java.util.Set[str]: ...
    def getPolicyQualifiersRejected(self) -> bool: ...
    def getSigProvider(self) -> str: ...
    def getTargetCertConstraints(self) -> CertSelector: ...
    def getTrustAnchors(self) -> java.util.Set[TrustAnchor]: ...
    def isAnyPolicyInhibited(self) -> bool: ...
    def isExplicitPolicyRequired(self) -> bool: ...
    def isPolicyMappingInhibited(self) -> bool: ...
    def isRevocationEnabled(self) -> bool: ...
    def setAnyPolicyInhibited(self, boolean: bool) -> None: ...
    def setCertPathCheckers(self, list: java.util.List[PKIXCertPathChecker]) -> None: ...
    def setCertStores(self, list: java.util.List[CertStore]) -> None: ...
    def setDate(self, date: java.util.Date) -> None: ...
    def setExplicitPolicyRequired(self, boolean: bool) -> None: ...
    def setInitialPolicies(self, set: java.util.Set[str]) -> None: ...
    def setPolicyMappingInhibited(self, boolean: bool) -> None: ...
    def setPolicyQualifiersRejected(self, boolean: bool) -> None: ...
    def setRevocationEnabled(self, boolean: bool) -> None: ...
    def setSigProvider(self, string: str) -> None: ...
    def setTargetCertConstraints(self, certSelector: CertSelector) -> None: ...
    def setTrustAnchors(self, set: java.util.Set[TrustAnchor]) -> None: ...
    def toString(self) -> str: ...

class URICertStoreParameters(CertStoreParameters):
    def __init__(self, uRI: java.net.URI): ...
    def clone(self) -> 'URICertStoreParameters': ...
    def equals(self, object: typing.Any) -> bool: ...
    def getURI(self) -> java.net.URI: ...
    def hashCode(self) -> int: ...
    def toString(self) -> str: ...

class X509CRL(CRL, X509Extension):
    def equals(self, object: typing.Any) -> bool: ...
    def getEncoded(self) -> typing.List[int]: ...
    def getIssuerDN(self) -> java.security.Principal: ...
    def getIssuerX500Principal(self) -> javax.security.auth.x500.X500Principal: ...
    def getNextUpdate(self) -> java.util.Date: ...
    @typing.overload
    def getRevokedCertificate(self, bigInteger: java.math.BigInteger) -> 'X509CRLEntry': ...
    @typing.overload
    def getRevokedCertificate(self, x509Certificate: 'X509Certificate') -> 'X509CRLEntry': ...
    def getRevokedCertificates(self) -> java.util.Set['X509CRLEntry']: ...
    def getSigAlgName(self) -> str: ...
    def getSigAlgOID(self) -> str: ...
    def getSigAlgParams(self) -> typing.List[int]: ...
    def getSignature(self) -> typing.List[int]: ...
    def getTBSCertList(self) -> typing.List[int]: ...
    def getThisUpdate(self) -> java.util.Date: ...
    def getVersion(self) -> int: ...
    def hashCode(self) -> int: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, string: str) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, provider: java.security.Provider) -> None: ...

class X509CRLEntry(X509Extension):
    def __init__(self): ...
    def equals(self, object: typing.Any) -> bool: ...
    def getCertificateIssuer(self) -> javax.security.auth.x500.X500Principal: ...
    def getEncoded(self) -> typing.List[int]: ...
    def getRevocationDate(self) -> java.util.Date: ...
    def getRevocationReason(self) -> CRLReason: ...
    def getSerialNumber(self) -> java.math.BigInteger: ...
    def hasExtensions(self) -> bool: ...
    def hashCode(self) -> int: ...
    def toString(self) -> str: ...

class X509CRLSelector(CRLSelector):
    def __init__(self): ...
    def addIssuer(self, x500Principal: javax.security.auth.x500.X500Principal) -> None: ...
    @typing.overload
    def addIssuerName(self, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def addIssuerName(self, string: str) -> None: ...
    def clone(self) -> typing.Any: ...
    def getCertificateChecking(self) -> 'X509Certificate': ...
    def getDateAndTime(self) -> java.util.Date: ...
    def getIssuerNames(self) -> java.util.Collection[typing.Any]: ...
    def getIssuers(self) -> java.util.Collection[javax.security.auth.x500.X500Principal]: ...
    def getMaxCRL(self) -> java.math.BigInteger: ...
    def getMinCRL(self) -> java.math.BigInteger: ...
    def match(self, cRL: CRL) -> bool: ...
    def setCertificateChecking(self, x509Certificate: 'X509Certificate') -> None: ...
    def setDateAndTime(self, date: java.util.Date) -> None: ...
    def setIssuerNames(self, collection: typing.Union[java.util.Collection[typing.Any], typing.Sequence[typing.Any], typing.Set[typing.Any]]) -> None: ...
    def setIssuers(self, collection: typing.Union[java.util.Collection[javax.security.auth.x500.X500Principal], typing.Sequence[javax.security.auth.x500.X500Principal], typing.Set[javax.security.auth.x500.X500Principal]]) -> None: ...
    def setMaxCRLNumber(self, bigInteger: java.math.BigInteger) -> None: ...
    def setMinCRLNumber(self, bigInteger: java.math.BigInteger) -> None: ...
    def toString(self) -> str: ...

class X509CertSelector(CertSelector):
    def __init__(self): ...
    @typing.overload
    def addPathToName(self, int: int, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def addPathToName(self, int: int, string: str) -> None: ...
    @typing.overload
    def addSubjectAlternativeName(self, int: int, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def addSubjectAlternativeName(self, int: int, string: str) -> None: ...
    def clone(self) -> typing.Any: ...
    def getAuthorityKeyIdentifier(self) -> typing.List[int]: ...
    def getBasicConstraints(self) -> int: ...
    def getCertificate(self) -> 'X509Certificate': ...
    def getCertificateValid(self) -> java.util.Date: ...
    def getExtendedKeyUsage(self) -> java.util.Set[str]: ...
    def getIssuer(self) -> javax.security.auth.x500.X500Principal: ...
    def getIssuerAsBytes(self) -> typing.List[int]: ...
    def getIssuerAsString(self) -> str: ...
    def getKeyUsage(self) -> typing.List[bool]: ...
    def getMatchAllSubjectAltNames(self) -> bool: ...
    def getNameConstraints(self) -> typing.List[int]: ...
    def getPathToNames(self) -> java.util.Collection[java.util.List[typing.Any]]: ...
    def getPolicy(self) -> java.util.Set[str]: ...
    def getPrivateKeyValid(self) -> java.util.Date: ...
    def getSerialNumber(self) -> java.math.BigInteger: ...
    def getSubject(self) -> javax.security.auth.x500.X500Principal: ...
    def getSubjectAlternativeNames(self) -> java.util.Collection[java.util.List[typing.Any]]: ...
    def getSubjectAsBytes(self) -> typing.List[int]: ...
    def getSubjectAsString(self) -> str: ...
    def getSubjectKeyIdentifier(self) -> typing.List[int]: ...
    def getSubjectPublicKey(self) -> java.security.PublicKey: ...
    def getSubjectPublicKeyAlgID(self) -> str: ...
    def match(self, certificate: Certificate) -> bool: ...
    def setAuthorityKeyIdentifier(self, byteArray: typing.List[int]) -> None: ...
    def setBasicConstraints(self, int: int) -> None: ...
    def setCertificate(self, x509Certificate: 'X509Certificate') -> None: ...
    def setCertificateValid(self, date: java.util.Date) -> None: ...
    def setExtendedKeyUsage(self, set: java.util.Set[str]) -> None: ...
    @typing.overload
    def setIssuer(self, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def setIssuer(self, string: str) -> None: ...
    @typing.overload
    def setIssuer(self, x500Principal: javax.security.auth.x500.X500Principal) -> None: ...
    def setKeyUsage(self, booleanArray: typing.List[bool]) -> None: ...
    def setMatchAllSubjectAltNames(self, boolean: bool) -> None: ...
    def setNameConstraints(self, byteArray: typing.List[int]) -> None: ...
    def setPathToNames(self, collection: typing.Union[java.util.Collection[java.util.List[typing.Any]], typing.Sequence[java.util.List[typing.Any]], typing.Set[java.util.List[typing.Any]]]) -> None: ...
    def setPolicy(self, set: java.util.Set[str]) -> None: ...
    def setPrivateKeyValid(self, date: java.util.Date) -> None: ...
    def setSerialNumber(self, bigInteger: java.math.BigInteger) -> None: ...
    @typing.overload
    def setSubject(self, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def setSubject(self, string: str) -> None: ...
    @typing.overload
    def setSubject(self, x500Principal: javax.security.auth.x500.X500Principal) -> None: ...
    def setSubjectAlternativeNames(self, collection: typing.Union[java.util.Collection[java.util.List[typing.Any]], typing.Sequence[java.util.List[typing.Any]], typing.Set[java.util.List[typing.Any]]]) -> None: ...
    def setSubjectKeyIdentifier(self, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def setSubjectPublicKey(self, byteArray: typing.List[int]) -> None: ...
    @typing.overload
    def setSubjectPublicKey(self, publicKey: java.security.PublicKey) -> None: ...
    def setSubjectPublicKeyAlgID(self, string: str) -> None: ...
    def toString(self) -> str: ...

class X509Certificate(Certificate, X509Extension):
    @typing.overload
    def checkValidity(self) -> None: ...
    @typing.overload
    def checkValidity(self, date: java.util.Date) -> None: ...
    def getBasicConstraints(self) -> int: ...
    def getExtendedKeyUsage(self) -> java.util.List[str]: ...
    def getIssuerAlternativeNames(self) -> java.util.Collection[java.util.List[typing.Any]]: ...
    def getIssuerDN(self) -> java.security.Principal: ...
    def getIssuerUniqueID(self) -> typing.List[bool]: ...
    def getIssuerX500Principal(self) -> javax.security.auth.x500.X500Principal: ...
    def getKeyUsage(self) -> typing.List[bool]: ...
    def getNotAfter(self) -> java.util.Date: ...
    def getNotBefore(self) -> java.util.Date: ...
    def getSerialNumber(self) -> java.math.BigInteger: ...
    def getSigAlgName(self) -> str: ...
    def getSigAlgOID(self) -> str: ...
    def getSigAlgParams(self) -> typing.List[int]: ...
    def getSignature(self) -> typing.List[int]: ...
    def getSubjectAlternativeNames(self) -> java.util.Collection[java.util.List[typing.Any]]: ...
    def getSubjectDN(self) -> java.security.Principal: ...
    def getSubjectUniqueID(self) -> typing.List[bool]: ...
    def getSubjectX500Principal(self) -> javax.security.auth.x500.X500Principal: ...
    def getTBSCertificate(self) -> typing.List[int]: ...
    def getVersion(self) -> int: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, string: str) -> None: ...
    @typing.overload
    def verify(self, publicKey: java.security.PublicKey, provider: java.security.Provider) -> None: ...

class PKIXBuilderParameters(PKIXParameters):
    @typing.overload
    def __init__(self, keyStore: java.security.KeyStore, certSelector: CertSelector): ...
    @typing.overload
    def __init__(self, set: java.util.Set[TrustAnchor], certSelector: CertSelector): ...
    def getMaxPathLength(self) -> int: ...
    def setMaxPathLength(self, int: int) -> None: ...
    def toString(self) -> str: ...

class PKIXCertPathBuilderResult(PKIXCertPathValidatorResult, CertPathBuilderResult):
    def __init__(self, certPath: CertPath, trustAnchor: TrustAnchor, policyNode: PolicyNode, publicKey: java.security.PublicKey): ...
    def getCertPath(self) -> CertPath: ...
    def toString(self) -> str: ...

class PKIXRevocationChecker(PKIXCertPathChecker):
    def clone(self) -> 'PKIXRevocationChecker': ...
    def getOcspExtensions(self) -> java.util.List[Extension]: ...
    def getOcspResponder(self) -> java.net.URI: ...
    def getOcspResponderCert(self) -> X509Certificate: ...
    def getOcspResponses(self) -> java.util.Map[X509Certificate, typing.List[int]]: ...
    def getOptions(self) -> java.util.Set['PKIXRevocationChecker.Option']: ...
    def getSoftFailExceptions(self) -> java.util.List['CertPathValidatorException']: ...
    def setOcspExtensions(self, list: java.util.List[Extension]) -> None: ...
    def setOcspResponder(self, uRI: java.net.URI) -> None: ...
    def setOcspResponderCert(self, x509Certificate: X509Certificate) -> None: ...
    def setOcspResponses(self, map: typing.Union[java.util.Map[X509Certificate, typing.List[int]], typing.Mapping[X509Certificate, typing.List[int]]]) -> None: ...
    def setOptions(self, set: java.util.Set['PKIXRevocationChecker.Option']) -> None: ...
    class Option(java.lang.Enum['PKIXRevocationChecker.Option']):
        ONLY_END_ENTITY: typing.ClassVar['PKIXRevocationChecker.Option'] = ...
        PREFER_CRLS: typing.ClassVar['PKIXRevocationChecker.Option'] = ...
        NO_FALLBACK: typing.ClassVar['PKIXRevocationChecker.Option'] = ...
        SOFT_FAIL: typing.ClassVar['PKIXRevocationChecker.Option'] = ...
        _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
        @typing.overload
        @staticmethod
        def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
        @typing.overload
        @staticmethod
        def valueOf(string: str) -> 'PKIXRevocationChecker.Option': ...
        @staticmethod
        def values() -> typing.List['PKIXRevocationChecker.Option']: ...

class CertPathValidatorException(java.security.GeneralSecurityException):
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable, certPath: CertPath, int: int): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable, certPath: CertPath, int: int, reason: 'CertPathValidatorException.Reason'): ...
    @typing.overload
    def __init__(self, throwable: java.lang.Throwable): ...
    def getCertPath(self) -> CertPath: ...
    def getIndex(self) -> int: ...
    def getReason(self) -> 'CertPathValidatorException.Reason': ...
    class BasicReason(java.lang.Enum['CertPathValidatorException.BasicReason'], java.security.cert.CertPathValidatorException.Reason):
        UNSPECIFIED: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        EXPIRED: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        NOT_YET_VALID: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        REVOKED: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        UNDETERMINED_REVOCATION_STATUS: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        INVALID_SIGNATURE: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        ALGORITHM_CONSTRAINED: typing.ClassVar['CertPathValidatorException.BasicReason'] = ...
        _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
        @typing.overload
        @staticmethod
        def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
        @typing.overload
        @staticmethod
        def valueOf(string: str) -> 'CertPathValidatorException.BasicReason': ...
        @staticmethod
        def values() -> typing.List['CertPathValidatorException.BasicReason']: ...
    class Reason(java.io.Serializable): ...

class PKIXReason(java.lang.Enum['PKIXReason'], CertPathValidatorException.Reason):
    NAME_CHAINING: typing.ClassVar['PKIXReason'] = ...
    INVALID_KEY_USAGE: typing.ClassVar['PKIXReason'] = ...
    INVALID_POLICY: typing.ClassVar['PKIXReason'] = ...
    NO_TRUST_ANCHOR: typing.ClassVar['PKIXReason'] = ...
    UNRECOGNIZED_CRIT_EXT: typing.ClassVar['PKIXReason'] = ...
    NOT_CA_CERT: typing.ClassVar['PKIXReason'] = ...
    PATH_TOO_LONG: typing.ClassVar['PKIXReason'] = ...
    INVALID_NAME: typing.ClassVar['PKIXReason'] = ...
    _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
    @typing.overload
    @staticmethod
    def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
    @typing.overload
    @staticmethod
    def valueOf(string: str) -> 'PKIXReason': ...
    @staticmethod
    def values() -> typing.List['PKIXReason']: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("java.security.cert")``.

    CRL: typing.Type[CRL]
    CRLException: typing.Type[CRLException]
    CRLReason: typing.Type[CRLReason]
    CRLSelector: typing.Type[CRLSelector]
    CertPath: typing.Type[CertPath]
    CertPathBuilder: typing.Type[CertPathBuilder]
    CertPathBuilderException: typing.Type[CertPathBuilderException]
    CertPathBuilderResult: typing.Type[CertPathBuilderResult]
    CertPathBuilderSpi: typing.Type[CertPathBuilderSpi]
    CertPathChecker: typing.Type[CertPathChecker]
    CertPathParameters: typing.Type[CertPathParameters]
    CertPathValidator: typing.Type[CertPathValidator]
    CertPathValidatorException: typing.Type[CertPathValidatorException]
    CertPathValidatorResult: typing.Type[CertPathValidatorResult]
    CertPathValidatorSpi: typing.Type[CertPathValidatorSpi]
    CertSelector: typing.Type[CertSelector]
    CertStore: typing.Type[CertStore]
    CertStoreException: typing.Type[CertStoreException]
    CertStoreParameters: typing.Type[CertStoreParameters]
    CertStoreSpi: typing.Type[CertStoreSpi]
    Certificate: typing.Type[Certificate]
    CertificateEncodingException: typing.Type[CertificateEncodingException]
    CertificateException: typing.Type[CertificateException]
    CertificateExpiredException: typing.Type[CertificateExpiredException]
    CertificateFactory: typing.Type[CertificateFactory]
    CertificateFactorySpi: typing.Type[CertificateFactorySpi]
    CertificateNotYetValidException: typing.Type[CertificateNotYetValidException]
    CertificateParsingException: typing.Type[CertificateParsingException]
    CertificateRevokedException: typing.Type[CertificateRevokedException]
    CollectionCertStoreParameters: typing.Type[CollectionCertStoreParameters]
    Extension: typing.Type[Extension]
    LDAPCertStoreParameters: typing.Type[LDAPCertStoreParameters]
    PKIXBuilderParameters: typing.Type[PKIXBuilderParameters]
    PKIXCertPathBuilderResult: typing.Type[PKIXCertPathBuilderResult]
    PKIXCertPathChecker: typing.Type[PKIXCertPathChecker]
    PKIXCertPathValidatorResult: typing.Type[PKIXCertPathValidatorResult]
    PKIXParameters: typing.Type[PKIXParameters]
    PKIXReason: typing.Type[PKIXReason]
    PKIXRevocationChecker: typing.Type[PKIXRevocationChecker]
    PolicyNode: typing.Type[PolicyNode]
    PolicyQualifierInfo: typing.Type[PolicyQualifierInfo]
    TrustAnchor: typing.Type[TrustAnchor]
    URICertStoreParameters: typing.Type[URICertStoreParameters]
    X509CRL: typing.Type[X509CRL]
    X509CRLEntry: typing.Type[X509CRLEntry]
    X509CRLSelector: typing.Type[X509CRLSelector]
    X509CertSelector: typing.Type[X509CertSelector]
    X509Certificate: typing.Type[X509Certificate]
    X509Extension: typing.Type[X509Extension]



import java.lang
import scala
import sigmastate.lang
import typing



class SigmaException(java.lang.Exception):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...
    def cause(self) -> scala.Option[java.lang.Throwable]: ...
    def getMessage(self) -> java.lang.String: ...
    def message(self) -> java.lang.String: ...
    def source(self) -> scala.Option[sigmastate.lang.SourceContext]: ...

class BinderException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class BuilderException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class CostLimitException(SigmaException):
    def __init__(self, estimatedCost: int, message: typing.Union[java.lang.String, str], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...
    def estimatedCost(self) -> int: ...
    @staticmethod
    def msgCostLimitError(cost: int, limit: int) -> java.lang.String: ...

class CosterException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class InterpreterException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class SerializerException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class SpecializerException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class TyperException(SigmaException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class ConstraintFailed(BuilderException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class DeserializeCallDepthExceeded(SerializerException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class InputSizeLimitExceeded(SerializerException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class InvalidArguments(BinderException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class InvalidBinaryOperationParameters(TyperException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class InvalidOpCode(SerializerException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class InvalidTypePrefix(SerializerException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext], cause: scala.Option[java.lang.Throwable]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...
    @staticmethod
    def $lessinit$greater$default$3() -> scala.Option[java.lang.Throwable]: ...

class InvalidUnaryOperationParameters(TyperException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class MethodNotFound(TyperException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...

class NonApplicableMethod(TyperException):
    def __init__(self, message: typing.Union[java.lang.String, str], source: scala.Option[sigmastate.lang.SourceContext]): ...
    @staticmethod
    def $lessinit$greater$default$2() -> scala.Option[sigmastate.lang.SourceContext]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("sigmastate.lang.exceptions")``.

    BinderException: typing.Type[BinderException]
    BuilderException: typing.Type[BuilderException]
    ConstraintFailed: typing.Type[ConstraintFailed]
    CostLimitException: typing.Type[CostLimitException]
    CosterException: typing.Type[CosterException]
    DeserializeCallDepthExceeded: typing.Type[DeserializeCallDepthExceeded]
    InputSizeLimitExceeded: typing.Type[InputSizeLimitExceeded]
    InterpreterException: typing.Type[InterpreterException]
    InvalidArguments: typing.Type[InvalidArguments]
    InvalidBinaryOperationParameters: typing.Type[InvalidBinaryOperationParameters]
    InvalidOpCode: typing.Type[InvalidOpCode]
    InvalidTypePrefix: typing.Type[InvalidTypePrefix]
    InvalidUnaryOperationParameters: typing.Type[InvalidUnaryOperationParameters]
    MethodNotFound: typing.Type[MethodNotFound]
    NonApplicableMethod: typing.Type[NonApplicableMethod]
    SerializerException: typing.Type[SerializerException]
    SigmaException: typing.Type[SigmaException]
    SpecializerException: typing.Type[SpecializerException]
    TyperException: typing.Type[TyperException]
